#include "AnalogModule.h"
#include "WPIStatus.h"
#include "AnalogChannel.h"
#include "Resource.h"
#include "Timer.h"
#include "Utility.h"
#include <math.h>

static SEM_ID semaphore = semMCreate(SEM_DELETE_SAFE | SEM_INVERSION_SAFE); // synchronize access to multi-value registers

/**
 * Get an instance of an Analog Module.
 * 
 * Singleton analog module creation where a module is allocated on the first use
 * and the same module is returned on subsequent uses.
 * 
 * @param slot The physical slot in the cRIO chassis where this analog module is installed.
 * @return A pointer to the AnalogModule.
 */
AnalogModule* AnalogModule::GetInstance(unsigned slot)
{
	CheckAnalogModule(slot);
	if (m_modules[slot] == NULL)
	{
		m_modules[slot] = new AnalogModule(slot);
	}
	return (AnalogModule*)m_modules[slot]; 
}

/**
 * Convert slot number to index.
 * 
 * @param slot The slot in the chassis where the module is plugged in.
 * @return An index to represent the module internally.
 */
unsigned AnalogModule::SlotToIndex(unsigned slot)
{
	return slot - 1;
}

/**
 * Create a new instance of an analog module.
 * 
 * Create an instance of the analog module object. Initialize all the parameters
 * to reasonable values on start.
 * Setting a global value on an analog module can be done only once unless subsequent
 * values are set the previously set value.
 * Analog modules are a singleton, so the constructor is never called outside of this class.
 * 
 * @param slot The slot in the chassis that the module is plugged into.
 */
AnalogModule::AnalogModule(unsigned slot)
   : Module(slot)
   , m_module (NULL)
   , m_sampleRateSet (false)
   , m_accumulatorOffset (0)
   , m_numChannelsToActivate (0)
{
	status = 0;
	AddToSingletonList();
	m_module = new tAI(SlotToIndex(slot), &status);
	SetNumChannelsToActivate(kAnalogChannels);
	SetSampleRate(kDefaultSampleRate);

	for (unsigned i = 0; i < kAnalogChannels; i++)
	{
		m_module->writeScanList(i, i, &status);
		SetAverageBits(i + 1, kDefaultAverageBits);
		SetOversampleBits(i + 1, kDefaultOversampleBits);
	}

	while ( ! m_module->readCalOK(&status) )
		Wait(1);

	m_module->strobeAccumulatorReset(&status);

	wpi_assertCleanStatus(status);
}

/**
 * Destructor for AnalogModule.
 */
AnalogModule::~AnalogModule()
{
   delete m_module;
   m_modules[m_slot] = NULL;
}

/**
 * Set the sample rate on the module.
 * 
 * This is a global setting for the module and effects all channels.
 * 
 * @param samplesPerSecond The number of samples per channel per second.
 */
void AnalogModule::SetSampleRate(float samplesPerSecond)
{
   // TODO: This will change when variable size scan lists are implemented.
   // TODO: Need float comparison with epsilon.
	//wpi_assert(!sampleRateSet || GetSampleRate() == samplesPerSecond);
	m_sampleRateSet = true;

	// Compute the convert rate
	unsigned ticksPerSample = (unsigned)((float)kTimebase / samplesPerSecond);
	unsigned ticksPerConversion = ticksPerSample / GetNumChannelsToActivate();
	// ticksPerConversion must be at least 80
	if (ticksPerConversion < 80)
	{
		wpi_fatal(SampleRateTooHigh);
		ticksPerConversion = 80;
	}

   // Atomically set the scan size and the convert rate so that the sample rate is constant
	tAI::tConfig config;
   config.ScanSize = GetNumChannelsToActivate();
   config.ConvertRate = ticksPerConversion;
   m_module->writeConfig(config, &status);

	// Indicate that the scan size has been commited to hardware.
   SetNumChannelsToActivate(0);

	wpi_assertCleanStatus(status);
}

/**
 * Get the current sample rate on the module.
 * 
 * This assumes one entry in the scan list.
 * This is a global setting for the module and effects all channels.
 * 
 * @return Sample rate.
 */
float AnalogModule::GetSampleRate(void)
{
   unsigned ticksPerConversion = m_module->readLoopTiming(&status);
   wpi_assertCleanStatus(status);
   unsigned ticksPerSample = ticksPerConversion * GetNumActiveChannels();
   wpi_assertCleanStatus(status);
   return (float)kTimebase / (float)ticksPerSample;
}

/**
 * Return the number of channels on the module in use.
 * 
 * @return Active channels.
 */
unsigned AnalogModule::GetNumActiveChannels(void)
{
   unsigned scanSize = m_module->readConfig_ScanSize(&status);
   wpi_assertCleanStatus(status);
   if (scanSize == 0)
	   return 8;
   return scanSize;
}

/**
 * Get the number of active channels.
 * 
 * This is an internal function to allow the atomic update of both the 
 * number of active channels and the sample rate.
 * 
 * When the number of channels changes, use the new value.  Otherwise,
 * return the curent value.
 * 
 * @return Value to write to the active channels field.
 */
unsigned AnalogModule::GetNumChannelsToActivate(void)
{
   if(m_numChannelsToActivate == 0) return GetNumActiveChannels();
   return m_numChannelsToActivate;
}

/**
 * Set the number of active channels.
 * 
 * Store the number of active channels to set.  Don't actually commit to hardware
 * until SetSampleRate().
 * 
 * @param channels Number of active channels.
 */
void AnalogModule::SetNumChannelsToActivate(unsigned channels)
{
	m_numChannelsToActivate = channels;
}

/**
 * Set the number of averaging bits.
 * 
 * This sets the number of averaging bits. The actual number of averaged samples is 2**bits.
 * Use averaging to improve the stability of your measurement at the expense of sampling rate.
 * The averaging is done automatically in the FPGA.
 * 
 * @param channel Analog channel to configure.
 * @param bits Number of bits to average.
 */
void AnalogModule::SetAverageBits(unsigned channel, unsigned bits)
{
	m_module->writeAverageBits(channel - 1, bits, &status);
   wpi_assertCleanStatus(status);
}

/**
 * Get the number of averaging bits.
 * 
 * This gets the number of averaging bits from the FPGA. The actual number of averaged samples is 2**bits.
 * The averaging is done automatically in the FPGA.
 * 
 * @param channel Channel to address.
 * @return Bits to average.
 */
unsigned AnalogModule::GetAverageBits(unsigned channel)
{
	unsigned result = m_module->readAverageBits(channel - 1, &status);
	wpi_assertCleanStatus(status);
   return result;
}

/**
 * Set the number of oversample bits.
 * 
 * This sets the number of oversample bits. The actual number of oversampled values is 2**bits.
 * Use oversampling to improve the resolution of your measurements at the expense of sampling rate.
 * The oversampling is done automatically in the FPGA.
 * 
 * @param channel Analog channel to configure.
 * @param bits Number of bits to oversample.
 */
void AnalogModule::SetOversampleBits(unsigned channel, unsigned bits)
{
	m_module->writeOversampleBits(channel - 1, bits, &status);
   wpi_assertCleanStatus(status);
}

/**
 * Get the number of oversample bits.
 * 
 * This gets the number of oversample bits from the FPGA. The actual number of oversampled values is
 * 2**bits. The oversampling is done automatically in the FPGA.
 * 
 * @param channel Channel to address.
 * @return Bits to oversample.
 */
unsigned AnalogModule::GetOversampleBits(unsigned channel)
{
	unsigned result = m_module->readOversampleBits(channel - 1, &status);
	wpi_assertCleanStatus(status);
   return result;
}

/**
 * Get a sample straight from the channel on this module.
 * 
 * The sample is a 12-bit value representing the -10V to 10V range of the A/D converter in the module.
 * The units are in A/D converter codes.  Use GetVoltage() to get the analog value in calibrated units.
 * 
 * @return A sample straight from the channel on this module.
 */
signed short AnalogModule::GetValue(unsigned channel)
{
	signed short value;
	CheckAnalogChannel(channel);

	tAI::tReadSelect readSelect;
	readSelect.Channel = channel - 1;
	readSelect.Module = SlotToIndex(m_slot);
	readSelect.Averaged = false;

	semTake(semaphore, WAIT_FOREVER);
	{
		m_module->writeReadSelect(readSelect, &status);
		m_module->strobeLatchOutput(&status);
		value = (signed short) m_module->readOutput(&status);
	}
	semGive(semaphore);

	wpi_assertCleanStatus(status);
	return value;
}

/**
 * Get a sample from the output of the oversample and average engine for the channel.
 * 
 * The sample is 12-bit + the value configured in SetOversampleBits().
 * The value configured in SetAverageBits() will cause this value to be averaged 2**bits number of samples.
 * This is not a sliding window.  The sample will not change until 2**(OversamplBits + AverageBits) samples
 * have been acquired from the module on this channel.
 * Use GetAverageVoltage() to get the analog value in calibrated units.
 * 
 * @param channel Channel number to read.
 * @return A sample from the oversample and average engine for the channel.
 */
signed int AnalogModule::GetAverageValue(unsigned channel)
{
	signed int value;
	CheckAnalogChannel(channel);

	tAI::tReadSelect readSelect;
	readSelect.Channel = channel - 1;
	readSelect.Module = SlotToIndex(m_slot);
	readSelect.Averaged = true;

	semTake(semaphore, WAIT_FOREVER); // link this instance on the list
	{
		m_module->writeReadSelect(readSelect, &status);
		m_module->strobeLatchOutput(&status);
		value = m_module->readOutput(&status);
	}
	semGive(semaphore);

	wpi_assertCleanStatus(status);
	return value;
}

/**
 * Convert a voltage to a raw value for a specified channel.
 * 
 * This process depends on the calibration of each channel, so the channel
 * must be specified.
 * 
 * @todo This assumes raw values.  Oversampling not supported as is.
 * 
 * @param channel The channel to convert for.
 * @param voltage The voltage to convert.
 * @return The raw value for the channel.
 */
signed int AnalogModule::VoltsToValue(int channel, float voltage)
{
	if (voltage > 10.0)
	{
		voltage = 10.0;
		wpi_fatal(VoltageOutOfRange);
	}
	if (voltage < -10.0)
	{
		voltage = -10.0;
		wpi_fatal(VoltageOutOfRange);
	}
	unsigned LSBWeight = m_module->readLSBWeight(channel - 1, &status);
	int offset = m_module->readOffset(channel - 1, &status);
	signed int value = (signed int) ((voltage + offset * 1.0e-9) / LSBWeight * 1.0e-9);
	wpi_assertCleanStatus(status);
	return value;
}

/**
 * Get a scaled sample straight from the channel on this module.
 * 
 * The value is scaled to units of Volts using the calibrated scaling data from GetLSBWeight() and GetOffset().
 * 
 * @param channel The channel to read.
 * @return A scaled sample straight from the channel on this module.
 */
float AnalogModule::GetVoltage(unsigned channel)
{
	signed short value = GetValue(channel);
	unsigned LSBWeight = m_module->readLSBWeight(channel - 1, &status);
	int offset = m_module->readOffset(channel - 1, &status);
	float voltage = LSBWeight * 1.0e-9 * value - offset * 1.0e-9;
	wpi_assertCleanStatus(status);
	return voltage;
}

/**
 * Get a scaled sample from the output of the oversample and average engine for the channel.
 * 
 * The value is scaled to units of Volts using the calibrated scaling data from GetLSBWeight() and GetOffset().
 * Using oversampling will cause this value to be higher resolution, but it will update more slowly.
 * Using averaging will cause this value to be more stable, but it will update more slowly.
 * 
 * @param channel The channel to read.
 * @return A scaled sample from the output of the oversample and average engine for the channel.
 */
float AnalogModule::GetAverageVoltage(unsigned channel)
{
	signed int value = GetAverageValue(channel);
	unsigned LSBWeight = m_module->readLSBWeight(channel - 1, &status);
	int offset = m_module->readOffset(channel - 1, &status);
	unsigned oversampleBits = GetOversampleBits(channel);
	float voltage = ((LSBWeight * 1.0e-9 * value) / (float)(1 << oversampleBits)) - offset * 1.0e-9;
	wpi_assertCleanStatus(status);
	return voltage;
}

/**
 * Get the factory scaling least significant bit weight constant.
 * The least significant bit weight constant for the channel that was calibrated in
 * manufacturing and stored in an eeprom in the module.
 * 
 * Volts = ((LSB_Weight * 1e-9) * raw) - (Offset * 1e-9)
 * 
 * @param channel The channel to get calibration data for.
 * @return Least significant bit weight.
 */
unsigned int AnalogModule::GetLSBWeight(unsigned channel) 
{
	// TODO: add scaling to make this worth while.
	// TODO: actually use this function.
	signed int lsbWeight = m_module->readLSBWeight(channel - 1, &status);
	wpi_assertCleanStatus(status);
	return lsbWeight;
}

/**
 * Get the factory scaling offset constant.
 * The offset constant for the channel that was calibrated in manufacturing and stored
 * in an eeprom in the module.
 * 
 * Volts = ((LSB_Weight * 1e-9) * raw) - (Offset * 1e-9)
 * 
 * @param channel The channel to get calibration data for.
 * @return Offset constant.
 */
signed int AnalogModule::GetOffset(unsigned channel)
{
	// TODO: add scaling to make this worth while.
	// TODO: actually use this function.
	signed int offset = m_module->readOffset(channel - 1, &status); 
	wpi_assertCleanStatus(status);
	return offset;
}


/**
 * Initialize the accumulator.
 */
void AnalogModule::InitAccumulator(void)
{
	m_accumulatorOffset = 0;
	SetAccumulatorCenter(0);
	ResetAccumulator();
}


/**
 * Set an inital value for the accumulator.
 * 
 * This will be added to all values returned to the user.
 * @param initialValue The value that the accumulator should start from when reset.
 */
void AnalogModule::SetAccumulatorInitialValue(long long initialValue)
{
	m_accumulatorOffset = initialValue;
}

/**
 * Resets the accumulator to the initial value.
 */
void AnalogModule::ResetAccumulator(void)
{
	m_module->strobeAccumulatorReset(&status);
	wpi_assertCleanStatus(status);
}

/**
 * Set the center value of the accumulator.
 * 
 * The center value is subtracted from each A/D value before it is added to the accumulator. This
 * is used for the center value of devices like gyros and accelerometers to make integration work
 * and to take the device offset into account when integrating.
 * 
 * This center value is based on the output of the oversampled and averaged source from channel 1.
 * Because of this, any non-zero oversample bits will affect the size of the value for this field.
 */
void AnalogModule::SetAccumulatorCenter(int center)
{
	m_module->writeAccumulatorCenter(center, &status);
	wpi_assertCleanStatus(status);
}

/**
 * Read the accumulated value.
 * 
 * Read the value that has been accumulating on channel 1.
 * The accumulator is attached after the oversample and average engine.
 * 
 * @return The 64-bit value accumulated since the last Reset().
 */
signed long long AnalogModule::GetAccumulatorValue(void)
{
	long long value = m_module->readAccumulatorOutput_Value(&status) + m_accumulatorOffset;
	wpi_assertCleanStatus(status);
	return value;
}

/**
 * Read the number of accumulated values.
 * 
 * Read the count of the accumulated values since the accumulator was last Reset().
 * 
 * @return The number of times samples from the channel were accumulated.
 */
unsigned AnalogModule::GetAccumulatorCount(void)
{
	status = 0;
	unsigned count = m_module->readAccumulatorOutput_Count(&status);
	wpi_assertCleanStatus(status);
	return count;
}


/**
 * Read the accumulated value and the number of accumulated values atomically.
 * 
 * This function reads the value and count from the FPGA atomically.
 * This can be used for averaging.
 * 
 * @param value Pointer to the 64-bit accumulated output.
 * @param count Pointer to the number of accumulation cycles.
 */
void AnalogModule::GetAccumulatorOutput(long long *value, unsigned *count)
{
	status = 0;
	if (value == NULL || count == NULL){
		wpi_fatal(NullParameter);
		return;
	}

	tAI::tAccumulatorOutput output = m_module->readAccumulatorOutput(&status);
	*value = output.Value + m_accumulatorOffset;
	*count = output.Count;
	wpi_assertCleanStatus(status);
}
