#include "WPILib.h"
#include <unistd.h>

/**
 * Initialize the gyro.
 * Calibrate the gyro by running for a number of samples and computing the center value for this
 * part. Then use the center value as the Accumulator center value for subsequent measurements.
 * It's imchannelant to make sure that the robot is not moving while the centering calculations are
 * in progress, this is typically done when the robot is first turned on while it's sitting at
 * rest before the competition starts.
 */
void Gyro::InitGyro()
{
	m_module = m_analog->GetModule();

	m_voltsPerDegreePerSecond = .0125;
	m_module = m_analog->GetModule();
	m_analog->SetAverageBits(kAverageBits);
	m_analog->SetOversampleBits(kOversampleBits);
	float sampleRate = kSamplesPerSecond * 
		(1 << (kAverageBits + kOversampleBits));
	m_module->SetSampleRate(sampleRate);
	Wait(1000);

	m_module->InitAccumulator();
	Wait(1000);

	long long value;
	unsigned count;
	m_module->GetAccumulatorOutput(&value, &count);

	unsigned center = (unsigned)((float)value / (float)count + .5);

	m_offset = ((float)value / (float)count) - (float)center;

	m_module->SetAccumulatorCenter(center);
	m_module->ResetAccumulator();
}

/**
 * Gyro constructor given a slot and a channel.
 * 
 * @param slot The cRIO slot for the analog module the gyro is connected to.
 * @param channel The analog channel the gyro is connected to.
 */
Gyro::Gyro(unsigned slot, unsigned channel)
{
	if (channel != AnalogModule::kAccumulatorChannel)
	{
		wpi_fatal(GyroNotAccumulatorChannel);
		channel = AnalogModule::kAccumulatorChannel;
	}
	m_analog = new AnalogChannel(slot, channel);
	m_channelAllocated = true;
	InitGyro();
}

/**
 * Gyro constructor with only a channel.
 * 
 * Use the default analog module slot.
 * 
 * @param channel The analog channel the gyro is connected to.
 */
Gyro::Gyro(unsigned channel)
{
	if (channel != AnalogModule::kAccumulatorChannel)
	{
		wpi_fatal(GyroNotAccumulatorChannel);
		channel = AnalogModule::kAccumulatorChannel;
	}
	m_analog = new AnalogChannel(channel);
	m_channelAllocated = true;
	InitGyro();
}

/**
 * Gyro constructor with a precreated analog channel object.
 * Use this constructor when the analog channel needs to be shared. There
 * is no reference counting when an AnalogChannel is passed to the gyro.
 * @param channel The AnalogChannel object that the gyro is connected to.
 */
Gyro::Gyro(AnalogChannel *channel)
{
	m_analog = channel;
	m_channelAllocated = false;
	if (channel == NULL)
	{
		wpi_fatal(NullParameter);
	}
	else
	{
		InitGyro();
	}
}

/**
 * Reset the gyro.
 * Resets the gyro to a heading of zero. This can be used if there is significant
 * drift in the gyro and it needs to be recalibrated after it has been running.
 */
void Gyro::Reset(void)
{
	m_module->ResetAccumulator();
}

/**
 * Delete (free) the accumulator and the analog components used for the gyro.
 */
Gyro::~Gyro(void)
{
	if (m_channelAllocated)
		delete m_analog;
}

/**
 * Return the actual angle in degrees that the robot is currently facing.
 * The angle is based on the current accumulator value corrected by the oversampling rate, the
 * gyro type and the A/D calibration values.
 * The angle is continuous, that is can go beyond 360 degrees. This make algorithms that wouldn't
 * want to see a discontinuity in the gyro output as it sweeps past 0 on the second time around.
 * 
 * @return the current heading of the robot in degrees. This heading is based on integration
 * of the returned rate from the gyro.
 */
float Gyro::GetAngle( void )
{
   long long rawValue;
   unsigned count;
   m_module->GetAccumulatorOutput(&rawValue, &count);

   long long value = rawValue - (long long)((float)count * m_offset);

   double scaledValue = value * 1e-9 * (double)m_analog->GetLSBWeight() * (double)(1 << m_analog->GetAverageBits()) /
         (m_module->GetSampleRate() * m_voltsPerDegreePerSecond);

   return (float)scaledValue;
}


/**
 * Set the gyro type based on the sensitivity.
 * This takes the number of volts/degree/second sensitivity of the gyro and uses it in subsequent
 * calculations to allow the code to work with multiple gyros.
 * 
 * @param voltsPerDegreePerSecond The type of gyro in degrees/volts.
 */
void Gyro::SetSensitivity( float voltsPerDegreePerSecond )
{
	m_voltsPerDegreePerSecond = voltsPerDegreePerSecond;
}

