#include "PWM.h"

#include "DigitalModule.h"
#include "Resource.h"
#include "Utility.h"

static Resource *allocated = NULL;

/**
 * Initialize PWMs given an module and channel.
 * This method is private and is the common path for all the constructors for creating PWM
 * instances. Check module and channel value ranges and allocate the appropriate channel. The allocation
 * is only done to help users ensure that they don't double assign channels.
 */
void PWM::InitPWM(unsigned slot, unsigned channel)
{
	Resource::CreateResourceObject(&allocated, tDIO::kNumSystems * kPwmChannels);
	CheckPWMModule(slot);
	CheckPWMChannel(channel);
	allocated->Allocate(DigitalModule::SlotToIndex(slot) * kPwmChannels + channel - 1);
	m_channel = channel;
	m_module = DigitalModule::GetInstance(slot);
	m_module->SetPWM(m_channel, kPwmDisabled);
}

/**
 * Allocate a PWM given a module and channel.
 * Allocate a PWM using a module and channel number.
 * 
 * @param slot The slot the digital module is plugged into.
 * @param channel The PWM channel on the digital module.
 */
PWM::PWM(unsigned slot, unsigned channel)
{
	InitPWM(slot, channel);
}

/**
 * Allocate a PWM in the default module give a channel.
 * Using a default module allocate a PWM given the channel number. The default module is the first
 * slot numerically in the cRIO chassis.
 * 
 * @param channel The PWM channel on the digital module.
 */
PWM::PWM(unsigned channel)
{
	InitPWM(GetDefaultDigitalModule(), channel);
}

/**
 * Free the PWM channel.
 * Free the resource associated with the PWM channel and set the value to 0.
 */
PWM::~PWM()
{
	m_module->SetPWM(m_channel, kPwmDisabled);
	allocated->Free(DigitalModule::SlotToIndex(m_module->GetSlot()) * kPwmChannels + m_channel - 1);
}

/**
 * Set the PWM value based on a position.
 * 
 * This is intended to be used by servos.
 * 
 * @pre SetMaxPositivePwm() called.
 * @pre SetMinNegativePwm() called.
 * 
 * @param pos The position to set the servo between 0.0 and 1.0.
 */
void PWM::SetPosition(float pos)
{
	if (pos < 0.0)
	{
		pos = 0.0;
	}
	else if (pos > 1.0)
	{
		pos = 1.0;
	}

	int rawValue;
	rawValue = (int)(pos * GetFullRangeScaleFactor() + m_minNegativePwm);

	wpi_assert((rawValue >= m_minNegativePwm) && (rawValue <= m_maxPositivePwm));
	wpi_assert(rawValue != kPwmDisabled);

	// send the computed pwm value to the FPGA
	SetRaw((unsigned char)rawValue);
}

/**
 * Get the PWM value in terms of a position.
 * 
 * This is intended to be used by servos.
 * 
 * @pre SetMaxPositivePwm() called.
 * @pre SetMinNegativePwm() called.
 * 
 * @return The position the servo is set to between 0.0 and 1.0.
 */
float PWM::GetPosition()
{
	int value = GetRaw();
	if (value < m_minNegativePwm)
	{
		return 0.0;
	}
	else if (value > m_maxPositivePwm)
	{
		return 1.0;
	}
	else
	{
		return (float)(value - m_minNegativePwm) / (float)GetFullRangeScaleFactor();
	}
}

/**
 * Set the PWM value based on a speed.
 * 
 * This is intended to be used by speed controllers.
 * 
 * @pre SetMaxPositivePwm() called.
 * @pre SetMinPositivePwm() called.
 * @pre SetCenterPwm() called.
 * @pre SetMaxNegativePwm() called.
 * @pre SetMinNegativePwm() called.
 * 
 * @param speed The speed to set the speed controller between -1.0 and 1.0.
 */
void PWM::SetSpeed(float speed)
{
	// clamp speed to be in the range 1.0 >= speed >= -1.0
	if (speed < -1.0)
	{
		speed = -1.0;
	}
	else if (speed > 1.0)
	{
		speed = 1.0;
	}

	// calculate the desired output pwm value by scaling the speed appropriately
	int rawValue;
	if (speed == 0.0)
	{
		rawValue = m_centerPwm;
	}
	else if (speed > 0.0)
	{
		rawValue = (int)(speed * ((float)GetPositiveScaleFactor()) +
									((float) m_minPositivePwm) + 0.5);
	}
	else
	{
		rawValue = (int)(speed * ((float)GetNegativeScaleFactor()) +
									((float) m_maxNegativePwm) + 0.5);
	}

	// the above should result in a pwm_value in the valid range
	wpi_assert((rawValue >= m_minNegativePwm) && (rawValue <= m_maxPositivePwm));
	wpi_assert(rawValue != kPwmDisabled);

	// send the computed pwm value to the FPGA
	SetRaw((unsigned char)rawValue);
}

/**
 * Get the PWM value in terms of speed.
 * 
 * This is intended to be used by speed controllers.
 * 
 * @pre SetMaxPositivePwm() called.
 * @pre SetMinPositivePwm() called.
 * @pre SetMaxNegativePwm() called.
 * @pre SetMinNegativePwm() called.
 * 
 * @return The most recently set speed between -1.0 and 1.0.
 */
float PWM::GetSpeed(void)
{
	int value = GetRaw();
	if (value > m_maxPositivePwm)
	{
		return 1.0;
	}
	else if (value < m_minNegativePwm)
	{
		return -1.0;
	}
	else if (value > m_minPositivePwm)
	{
		return (float)(value - m_minPositivePwm) / (float)GetPositiveScaleFactor();
	}
	else if (value < m_maxNegativePwm)
	{
		return (float)(value - m_maxNegativePwm) / (float)GetNegativeScaleFactor();
	}
	else
	{
		return 0.0;
	}
}

/**
 * Set the PWM value directly to the hardware.
 * 
 * Write a raw value to a PWM channel.
 * 
 * @param value Raw PWM value.  Range 0 - 255.
 */
void PWM::SetRaw(unsigned char value)
{
	m_module->SetPWM(m_channel, value);
}

/**
 * Get the PWM value directly from the hardware.
 * 
 * Read a raw value from a PWM channel.
 * 
 * @return Raw PWM control value.  Range: 0 - 255.
 */
unsigned char PWM::GetRaw(void)
{
	return m_module->GetPWM(m_channel);
}

/**
 * Slow down the PWM signal for old devices.
 * 
 * @param mult The period multiplier to apply to this channel
 */
void PWM::SetPeriodMultiplier(PeriodMultiplier mult)
{
	switch(mult)
	{
	case kPeriodMultiplier_4X:
		m_module->SetPWMPeriodScale(m_channel, 3); // Squelch 3 out of 4 outputs
		break;
	case kPeriodMultiplier_2X:
		m_module->SetPWMPeriodScale(m_channel, 1); // Squelch 1 out of 2 outputs
		break;
	case kPeriodMultiplier_1X:
		m_module->SetPWMPeriodScale(m_channel, 0); // Don't squelch any outputs
		break;
	default:
		wpi_assert(false);
	}
}

