#include "Watchdog.h"

#include "Utility.h"

/**
 * The Watchdog is born.
 */
Watchdog::Watchdog(void)
	:	m_fpgaWatchDog(NULL)
{
	m_fpgaWatchDog = new tWatchdog(&status);
	wpi_assertCleanStatus(status);
	SetExpiration(kDefaultWatchdogExpiration_ms);
	SetEnabled(true);
}

/**
 * Time to bury him in the back yard.
 */
Watchdog::~Watchdog(void)
{
	SetEnabled(false);
	delete m_fpgaWatchDog;
	m_fpgaWatchDog = NULL;
}

/**
 * Throw the dog a bone.
 * 
 * When everything is going well, you feed your dog when you get home.
 * Let's hope you don't drive your car off a bridge on the way home...
 * Your dog won't get fed and he will starve to death.
 * 
 * By the way, it's not cool to ask the neighbor (some random task) to
 * feed your dog for you.  He's your responsibility!
 */
void Watchdog::Feed(void)
{
	m_fpgaWatchDog->strobeUserFeed(&status);
	wpi_assertCleanStatus(status);
}

/**
 * Put the watchdog out of its misery.
 * 
 * Don't wait for your dying robot to starve when there is a problem.
 * Kill it quickly, cleanly, and humanely.
 */
void Watchdog::Kill(void)
{
	m_fpgaWatchDog->strobeUserKill(&status);
	wpi_assertCleanStatus(status);
}

/**
 * Read how long it has been since the watchdog was last fed.
 * 
 * @return The number of milliseconds since last meal.
 */
unsigned int Watchdog::GetTimer(void)
{
	unsigned int timer = m_fpgaWatchDog->readUserTimer(&status);
	wpi_assertCleanStatus(status);
	return timer / (kSystemClockTicksPerMicrosecond * 1000);
}

/**
 * Read what the current expiration is.
 * 
 * @return The number of milliseconds before starvation.
 */
unsigned int Watchdog::GetExpiration(void)
{
	unsigned int expiration = m_fpgaWatchDog->readUserExpiration(&status);
	wpi_assertCleanStatus(status);
	return expiration / (kSystemClockTicksPerMicrosecond * 1000);
}

/**
 * Configure how many milliseconds your watchdog can be neglected before he starves to death.
 * 
 * @param expiration_ms The number of milliseconds before starvation.
 */
void Watchdog::SetExpiration(unsigned int expiration_ms)
{
	m_fpgaWatchDog->writeUserExpiration(expiration_ms * (kSystemClockTicksPerMicrosecond * 1000), &status);
	wpi_assertCleanStatus(status);
}

/**
 * Find out if the watchdog is currently enabled or disabled (mortal or immortal).
 * 
 * @return Enabled or disabled.
 */
bool Watchdog::GetEnabled(void)
{
	bool enabled = m_fpgaWatchDog->readUserEnabled(&status);
	wpi_assertCleanStatus(status);
	return enabled;
}

/**
 * Enable or disable the watchdog timer.
 * 
 * When enabled, you must keep feeding the watchdog timer to
 * keep the watchdog active, and hence the dangerous parts 
 * (motor outputs, etc.) can keep functioning.
 * When disabled, the watchdog is immortal and will remain active
 * even without being fed.  It will also ignore any kill commands
 * while disabled.
 * 
 * @param enabled Enable or disable the watchdog.
 */
void Watchdog::SetEnabled(bool enabled)
{
	m_fpgaWatchDog->writeUserEnabled(enabled, &status);
	wpi_assertCleanStatus(status);
}

/**
 * Check in on the watchdog and make sure he's still kicking.
 * 
 * This indicates that your watchdog is allowing the system to operate.
 * It is still possible that the network communications is not allowing the
 * system to run, but you can check this to make sure it's not your fault.
 * Check IsSystemActive() for overall system status.
 * 
 * If the watchdog is disabled, then your watchdog is immortal.
 * 
 * @return Is the watchdog still alive?
 */
bool Watchdog::IsAlive(void)
{
	bool alive = m_fpgaWatchDog->readUserActive(&status);
	wpi_assertCleanStatus(status);
	return alive;
}

/**
 * Check on the overall status of the system.
 * 
 * @return Is the system active (i.e. PWM motor outputs, etc. enabled)?
 */
bool Watchdog::IsSystemActive(void)
{
	bool alive = m_fpgaWatchDog->readSystemActive(&status);
	wpi_assertCleanStatus(status);
	return alive;
}
