/*----------------------------------------------------------------------------*/
/* Copyright (c) FIRST 2008. All Rights Reserved.							  */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in $(WIND_BASE)/WPILib.  */
/*----------------------------------------------------------------------------*/

#include "PIDController.h"
#include "PID.h"

/**
 * Allocate a PID object with the given constants for P, I, D
 */
PIDController::PIDController(float p, float i, float d)
{
	controlLoop_ = new Notifier(PIDController::Calculate,this);
	
	P_ = p;
	I_ = i;
	D_ = d;
	maximumOutput_ = 1.0;
	minimumOutput_ = 0.0;
	
	maximumInput_ = 1024;
	minimumInput_ = 0;
	
	continuous_ = false;
	
	enabled_ = false;
	
	prevError_ = 0;
	
	totalError_ = 0;
	
	tolerence_ = .05;

	result_ = 0;
	
	controlLoop_->StartPeriodic(100);
}
	
/**
 * Free the PID object
 */
PIDController::~PIDController(void)
{
}
	
void PIDController::Calculate(void *controller)
{
	PIDController* control = (PIDController*)controller;
	long input = control->pidInput_->PIDGet();
	
	if (control->enabled_)
	{
		control->error_ = control->setpoint_ - input;
		if (control->continuous_)
		{
			if (abs(control->error_) > control->maximumInput_ - control->minimumInput_)
			{
				if (control->error_ > 0)
					control->error_ = control->error_  - control->maximumInput_ + control->minimumInput_;
				else
					control->error_ = control->error_  + control->maximumInput_ - control->minimumInput_;
			}
		}

		if (((control->totalError_ + control->error_) * control->I_ < control->maximumOutput_) ||
				((control->totalError_ + control->error_) * control->I_ > control->minimumOutput_))
			control->totalError_ += control->error_;

				
		control->result_ = control->P_ * control->error_ + 
			control->I_ * control->totalError_ +
			control->D_ * (control->error_ - control->prevError_);
		control->prevError_ = control->error_;
		
		if (control->result_ > control->maximumOutput_)
			control->result_ = control->maximumOutput_;
		else if (control->result_ < control->minimumOutput_)
			control->result_ = control->minimumOutput_;
		
		control->pidOutput_->PIDWrite(control->result_);
	}
}

/**
 * Return the current PID result
 * This is always centered on zero and constrained the the max and min outs
 */
float PIDController::Get(void)
{
	return result_;
}

/**
 *  Set the PID controller to consider the input to be continuous,
 *  Rather then using the max and min in as constraints, it considers them to
 *  be the same point and automatically calculates the shortest route to
 *  the setpoint.
 */
void PIDController::SetContinuous(void)
{
	continuous_ = true;
}

void PIDController::SetContinuous(bool continuous)
{
	continuous_ = continuous;
}

void PIDController::SetInput(PIDSource *pidInput)
{
	pidInput_ = pidInput;
}

void PIDController::SetInput(PIDSource *pidInput, long minimumInput, long maximumInput)
{
	pidInput_ = pidInput;
	minimumInput_ = minimumInput;
	maximumInput_ = maximumInput;	
}

void PIDController::SetOutput(PIDOutput *pidOutput)
{
	pidOutput_ = pidOutput;
}

void PIDController::SetOutput(PIDOutput *pidOutput, float minimumOutput, float maximumOutput)
{
	pidOutput_ = pidOutput;
	minimumOutput_ = minimumOutput;
	maximumOutput_ = maximumOutput;
}

void PIDController::SetSetpoint(long setpoint)
{
	if (setpoint > maximumInput_)
		setpoint_ = maximumInput_;
	else if (setpoint < minimumInput_)
		setpoint_ = minimumInput_;
	else
		setpoint = setpoint_;
}

long PIDController::GetSetpoint(long setpoint)
{
	return setpoint_;
}

long PIDController::GetError(void)
{
	return error_;
}

void PIDController::SetTolerence(float percent)
{
	tolerence_ = percent;
}

bool PIDController::OnTarget(void)
{
	return (abs(error_)<tolerence_ / 100 * (maximumInput_ - minimumInput_));
}

void PIDController::Enable(void)
{
	enabled_ = true;
}

void PIDController::Disable(void)
{
	enabled_ = false;
}

void PIDController::Reset(void)
{
	enabled_ = false;
	
	prevError_ = 0;
	
	totalError_ = 0;
	
	result_ = 0;
}
