/*----------------------------------------------------------------------------*/
/* Copyright (c) FIRST 2008. All Rights Reserved.							  */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in $(WIND_BASE)/WPILib.  */
/*----------------------------------------------------------------------------*/

#include "Task.h"

#include "Utility.h"
#include "WPIStatus.h"
#include <string.h>


extern INT32 ERRNO;

Task::Task(char* name, INT32 priority)
{
	m_priority = priority;
	m_taskName = new char[strlen(name) + 1];
	strcpy(m_taskName, name);
}

Task::~Task(void)
{
	delete [] m_taskName;
}

/**
 * Handles errors generated by task related code.
 */
void Task::HandleError()
{
	switch(ERRNO)
	{
	case S_objLib_OBJ_ID_ERROR:
		wpi_fatal(TaskIDError);
		break;
		
	case S_objLib_OBJ_DELETED:
		wpi_fatal(TaskDeletedError);
		break;
		
	case S_taskLib_ILLEGAL_OPTIONS:
		wpi_fatal(TaskOptionsError);
		break;
		
	case S_memLib_NOT_ENOUGH_MEMORY:
		wpi_fatal(TaskMemoryError);
		break;
		
	case S_taskLib_ILLEGAL_PRIORITY:
		wpi_fatal(TaskPriorityError);
		break;
		
	default:
		wpi_fatal(TaskError);
	}
	wpi_fatal(TaskError);
}

/**
 * Runs this task.
 * If it is already running or unable to start, it fails and returns false.
 */
bool Task::Run(FUNCPTR function, INT32 arg)
{
	m_taskID = taskSpawn(m_taskName,
						m_priority,
						VX_FP_TASK,							// options
						20000,								// stack size
						(FUNCPTR) function,					// function to start
						arg,								// parameter 1 - pointer to this class
						0, 0, 0, 0, 0, 0, 0, 0, 0);			// additional unused parameters
	if (m_taskID == ERROR)
	{
		HandleError();
		return false;
	}
	return true;
}

/**
 * Restarts a running task.
 * If the task isn't started, it starts it.
 * If the task is running and we are unable to kill the previous instance, returns false.
 */
bool Task::Restart(void)
{
	if (taskRestart(m_taskID) == ERROR)
	{
		HandleError();
		return false;
	}
	return true;
}

/**
 * Kills the running task.
 * Returns true on success false if the task doesn't exist or we are unable to kill it.
 */
bool Task::Delete(void)
{
	if (taskDelete(m_taskID) == ERROR)
	{
		HandleError();
		return false;
	}
	return false;
}

/**
 * Pauses a running task.
 * Returns true on success, false if unable to pause or the task isn't running.
 */
bool Task::Suspend(void)
{
	if (taskSuspend(m_taskID) == ERROR)
	{
		HandleError();
		return false;
	}
	return true;
}

/**
 * Resumes a paused task.
 * Returns true on success, false if unable to resume or if the task isn't running/paused.
 */
bool Task::Resume(void)
{
	if (taskResume(m_taskID) == ERROR)
	{
		HandleError();
		return false;
	}
	return true;
}

/**
 * Verifies a task is running.
 */
bool Task::Verify(void)
{
	if (taskIdVerify(m_taskID) == OK)
		return true;
	else
		return false;
}

/**
 * Gets the priority of a task.
 * Range: ? to ???
 */
INT32 Task::GetPriority(void)
{
	if (taskPriorityGet(m_taskID, &m_priority) == ERROR)
	{
		HandleError();
		return false;
	}
	return m_priority;
}

/**
 * Sets the priority of the task.
 * Range: ? to ???
 */
void Task::SetPriority(INT32 priority)
{
	m_priority = priority;
	if (taskPrioritySet(m_taskID, m_priority) == ERROR)
	{
		HandleError();
		return;
	}
}

/**
 * Returns the name of the task.
 */
char* Task::GetName(void)
{
	return m_taskName;
}

/**
 * Returns the task ID of this task.
 */
INT32 Task::GetID(void)
{
	return m_taskID;
}

