/********************************************************************************
*
*  Project   		: FIRST Motor Controller
*  Category 		: Team Deliverable Software (as source code) 	
*  Subsystem 		: FRC Vision for cRIO     			
* 
*  File Name  		: frctrack.cpp        
*  Contributors   	: ELF
*  Creation Date 	: August 10, 2008
*  Revision History	: Source code and revision history are maintained at
*                               	   sourceforge.WPI.edu  
*  
*  File Description	: Tracking Routines for FIRST Vision API
* 
*  API: Because nivision.h uses C++ style comments, any file including this
*  must be a .cpp (not .c)
*/ 
/*----------------------------------------------------------------------------*/
/*        Copyright (c) FIRST 2008.  All Rights Reserved.                     */
/*  Open Source Software - may be modified and shared by FRC teams. The code  */
/*  must be accompanied by the FIRST BSD license file in $(WIND_BASE)/WPILib. */
/*----------------------------------------------------------------------------*/

#include "vxWorks.h" 
#include "stdioLib.h" 
#include "string.h"

#include "frcvision.h" 
#include "frcvtrack.h" 
#include "frcvAxis.h" 
#include "baefrc_utilities.h"

int frcvtrack_debugFlag = 0;
#define DPRINTF if(frcvtrack_debugFlag)dprintf

/** image quality requirement: particle must be .0025 of pixels */
#define PARTICLE_TO_IMAGE_PERCENT 0.25

/**
* @brief Search for a color. Supports IMAQ_IMAGE_HSL. 
* @param color Definition for the hue range 
* @param trackReport Values for tracking: center of particle, particle size, color
* @return 0 = error
*/
int frcFindColor(FrcHue color, FrcParticleAnalysisReport* trackReport)
{
	char funcName[]="frcFindColor";
	int success = 0;		// return: 0 = error
	
	Range hueRange, satRange, lumRange;
	
	/* track color */
	frcSetHueRange(color, &hueRange);
	
	// use ACTIVE_LIGHT or WHITE_LIGHT for brightly lit objects
	frcSetLightLevel(PASSIVE_LIGHT, &satRange, &lumRange);

	success = frcFindColor(IMAQ_HSL, &hueRange, &satRange, &lumRange, trackReport); 
	if ( !success )	{ 
		//dprintf (LOG_INFO, "did not find color - errorCode= %i",frcGetLastError());	
		return success;
	}

	//frcPrintReport(par);
	
	/* set an image quality restriction */
	if (trackReport->particleToImagePercent < PARTICLE_TO_IMAGE_PERCENT) {
		imaqSetError(ERR_frcPARTICLE_TOO_SMALL, funcName);
		success = 0;
	}	
	return success;
}

/**
* @brief Search for a color. Supports IMAQ_IMAGE_HSL. 
* @param hueRange The range for the first plane
* @param trackReport Values for tracking: center of particle, particle size, color
* @return 0 = error
*/
int frcFindColor(const Range* hueRange, FrcParticleAnalysisReport *trackReport)
{ return frcFindColor(hueRange, frcDEFAULT_SATURATION_THRESHOLD, trackReport); }

/**
* @brief Search for a color. Supports IMAQ_IMAGE_HSL. 
* @param hueRange The range for the first plane
* @param minSaturation The lower range saturation
* @param trackReport Values for tracking: center of particle, particle size, color
* @return 0 = error
*/
int frcFindColor(const Range* hueRange, int minSaturation, FrcParticleAnalysisReport *trackReport)
{
	Range satRange;
	satRange.minValue = minSaturation;
	satRange.maxValue = 255;
	Range lumRange;
	lumRange.minValue = 0;
	lumRange.maxValue = 255;
	ColorMode cmode = IMAQ_HSL;
	return frcFindColor(cmode, hueRange, &satRange, &lumRange, trackReport);
}

/**
* @brief Search for a color. Supports IMAQ_IMAGE_HSL and IMAQ_IMAGE_RGB. 
* @param mode Color mode, either IMAQ_HSL or IMAQ_RGB
* @param plane1Range The range for the first plane (hue or red)
* @param plane2Range The range for the second plane (saturation or green)
* @param plane3Range The range for the third plane (luminance or blue)
* @param trackReport Values for tracking: center of particle, particle size, etc
* @return 0 = error
*/
int frcFindColor(ColorMode mode, const Range* plane1Range, const Range* plane2Range, 
		const Range* plane3Range, FrcParticleAnalysisReport *trackReport)
{
	return frcFindColor(mode, plane1Range, plane2Range, plane3Range, trackReport, NULL);
}

/**
* @brief Search for a color. Supports IMAQ_IMAGE_HSL and IMAQ_IMAGE_RGB. 
* @param mode Color mode, either IMAQ_HSL or IMAQ_RGB
* @param plane1Range The range for the first plane (hue or red)
* @param plane2Range The range for the second plane (saturation or green)
* @param plane3Range The range for the third plane (luminance or blue)
* @param trackReport Values for tracking: center of particle, particle size, etc
* @param colorReport Color charactaristics of the particle
* @return 0 = error
*/
int frcFindColor(ColorMode mode, const Range* plane1Range, const Range* plane2Range, 
		const Range* plane3Range, FrcParticleAnalysisReport *trackReport, 
		FrcColorReport *colorReport)
{
	char funcName[]="frcFindColor";
	int errorCode;
	int success = 0;
	
	/* create an image object */
	Image* cameraImage = frcCreateImage(IMAQ_IMAGE_HSL);
	if (!cameraImage)  { errorCode = frcGetLastError();  return errorCode; }
	
	/* get image from camera */
	int trycount = 0;
	int maxtries = 10;
	for (trycount=0; trycount<maxtries; trycount++)  {
		success = frcGetImage(cameraImage,NULL);
		if (success) {break;}	 
		frcSleep(0.02);
	}
	if (!success){
		DPRINTF(LOG_INFO, "No camera Image - giving up after %i tries\n Error = %i %s", 
				trycount, errorCode, frcGetErrorText(errorCode));
		frcDispose(cameraImage); 
		imaqSetError(errorCode, funcName);	//reset error code for the caller	
		return success;		
	}	
	
	/* save a copy of the image to another image for color thresholding later */
	Image* histImage = frcCreateImage(IMAQ_IMAGE_HSL);
	if (!histImage)  { frcDispose(cameraImage); return success; }
	success = frcCopyImage(histImage,cameraImage);
	if ( !success )	{ 
		errorCode = frcGetLastError(); 
		frcDispose(funcName,cameraImage,histImage,NULL); 
		return success; 
	}	
	
	/* Color threshold the image */
	success = frcColorThreshold(cameraImage, cameraImage, mode, plane1Range, plane2Range, plane3Range);
	if ( !success )	{ 
		errorCode = frcGetLastError(); 
		DPRINTF (LOG_DEBUG, "Error = %i  %s ", errorCode, frcGetErrorText(errorCode));
		frcDispose(funcName,cameraImage,histImage,NULL); 
		return success; 
	}	
	
	/* determine number of particles in thresholded image */	
	int numParticles;
	success = frcCountParticles(cameraImage, &numParticles);
	if ( !success )	{
		errorCode = frcGetLastError(); 
		DPRINTF (LOG_DEBUG, "Error = %i  %s ", errorCode, frcGetErrorText(errorCode));
		frcDispose(funcName,cameraImage,histImage,NULL); 
		return success; 
	}			
	DPRINTF(LOG_INFO, "number of particles = %i\n", numParticles);
	
	/* if no particles found we can quit here */
	if (numParticles > 0)  {
		// find the largest particle
		int largestParticleIndex = -1;
		double largestParticleArea = 0;
		double particleArea;
		for (int i = 0; i < numParticles; ++i) {		
			success = imaqMeasureParticle(cameraImage, i, 0, IMAQ_MT_AREA, &particleArea);
			if ( !success )	{ 
				errorCode = frcGetLastError(); 
				DPRINTF (LOG_DEBUG, "Error = %i  %s ", errorCode, frcGetErrorText(errorCode));
				frcDispose(funcName,cameraImage,histImage,NULL); 
				return success;
			}		
			if (particleArea > largestParticleArea) 	{
				largestParticleArea = particleArea;
				largestParticleIndex = i;
			}
		}
		
		/* 
		 * Fill in report information for largest particle found
		 */
		success = frcParticleAnalysis(cameraImage, largestParticleIndex, trackReport);
		if (!success) {frcDispose(funcName,cameraImage,histImage,NULL); return success;}
		
		/* particle color statistics */
		/* only if a color report requested */
		if (colorReport != NULL)
		{
			/* first filter out the other particles */
			ParticleFilterCriteria2 criteria;
			ParticleFilterOptions* options;
			Rect rect;
			int numParticles;
			success = frcParticleFilter(cameraImage, cameraImage, &criteria, 1, options, 
					rect, &numParticles);
			if ( !success )	{ 
				DPRINTF(LOG_INFO, "frcParticleFilter errorCode %i", frcGetLastError()); 
			}	
					
			/* histogram the original image using the thresholded image as a mask */
			int numClasses = 10; //how many classes?
			ColorHistogramReport* chrep = imaqColorHistogram2(histImage, numClasses, IMAQ_HSL, 
						NULL, cameraImage);
			if (chrep == NULL) { 
				DPRINTF(LOG_INFO, "NULL Color Histogram");
				errorCode = frcGetLastError(); 
			} else {
				colorReport->particleHueMax = chrep->plane1.max;
				colorReport->particleHueMin = chrep->plane1.min;
				colorReport->particleHueMean = chrep->plane1.mean;
				colorReport->particleSatMax = chrep->plane2.max;
				colorReport->particleSatMin  = chrep->plane2.min;
				colorReport->particleSatMean = chrep->plane2.mean;
				colorReport->particleLumMax = chrep->plane3.max;
				colorReport->particleLumMin = chrep->plane3.min;
				colorReport->particleLumMean = chrep->plane3.mean;
				colorReport->numberParticlesFound = numParticles;
				frcDispose(chrep); 
			}
		}

	}	else	{
		/* no particles found */
		imaqSetError(ERR_frcCOLOR_NOT_FOUND, funcName);
		success = 0;
	}
	/* clean up */
	frcDispose(funcName,cameraImage,histImage,NULL); 
	
	return success;	
}


/**
* @brief Find a color specified in a template. Supports IMAQ_IMAGE_RGB, IMAQ_IMAGE_HSL.
* @param template A color template image.
* @return 0 = error
*/
int frcFindColorFromTemplate(Image* templateImage)
{
	//char funcName[]="frcFindColorFromTemplate";
	//TODO: implement frcFindColorFromTemplate
	DPRINTF ("frcFindColorFromTemplate not yet implemented");
	// read in template
	// call color threshold on template
	// get color ranges of largest particle 
	// call frcFindColor	
	return 0;
}





